
/*

This is just an example of how mouse selection can work in BYOND

Feel free to make changes, test it out, or figure out how it works

Run it to test

*/


// Two very usefull libraries made by Deadron are included to help the demo

#include <deadron/eventloop>
#include <deadron/pathfinding>


world
	view = 6
	turf = /turf/default
	mob  = /mob/you



turf

	icon='Turf.dmi'

	default

	block
		icon_state = "2"
		density = 1



mob

// The following variables are used in movement of the units


	var/tmp
		destination          // The destination to try to reach
		last_destination     // Used in case of a bump
		next_walk_time       // When is the next time for us to walk?
		walking_delay = 5    // How long between movement?



	you  // Player's mob

		density = 0

		Login()

			src.loc = locate(12,9,1)
			src << "<b>Click and drag to select units, then click a destination.</b>"
			src << "<b>Double click while units are selected to move and automatically deselect them.</b>"
			src << "<b>Use the arrow keys to move your view.</b>"
			return ..()

	unit  // Units players can control

		icon='Unit.dmi'


		Bump(mob/O)
			if(ismob(O)&&O.destination)  // if we bump another unit and it has a destination -- wait
				sleep(src.walking_delay)
				src.destination = src.last_destination

		base_EventCycle()  // From Pathfinding demo

			// This gets called once each tick.

			if (next_walk_time <= world.time)

				// It's time for us to walk if given an order
				if (destination)

					var/reached_dest = base_StepTowards(destination)

					if (reached_dest)

						// We reached the destination or got as close as we could
						destination = null


				// Set the next walk time.
				next_walk_time = world.time + walking_delay

			return


client

	var/tmp

		select[]=list()  // list of selected units
		imageslist[]=list()  // list of selection images
		turf/start  // grid start
		turf/end  //grid end

	verb/deselect()

		// if something is selected
		if(select.len)

			// deselect it/them
			for(var/mob/unit/u in select)
				select -= u
			for(var/i in src.imageslist)
				del(i)

	Click(atom/movable/object, location)

		// if something is selected
		if(select.len)

			// give it/them a destination, if the destination is valid
			if(isloc(object.loc))

				world << "<i>Destination set to [object] at [object.x],[object.y]</i>"

				for(var/mob/unit/u in select)
					u.destination = object
					u.last_destination = u.destination

		..()

	DblClick(atom/movable/object, location)

		// an order is given on the first click

		src.deselect()  // now deselect units

	MouseDown(object,location)


		// if nothing is selected, draw the grid
		if(!select.len)

			src.drawGrid(location,location)

		//..()

	MouseUp(object,location)

		// if there is a selection box, select!
		if(start)

			endGrid()

		..()

	MouseDrag(src_object,over_object,src_location,over_location)

		// if there is a grid started, expand it
		if(start)

			drawGrid(start,over_location)

		..()

	proc/drawGrid(s,e)

		if(!isloc(s,e)) //if mouse is not at a valid loc return
			return

		// if there is no start location, set it
		if(!start)

			start = s

		end = e // current mouse location

		// delete old grid
		for(var/image/i in src.imageslist)

			del(i)

		// vars set to the difference between 2 corners of box
		var/xd = end.x-start.x
		var/yd = end.y-start.y

		// 4 corners of grid are given are displayed to the user
		src.imageslist += new/image ('Selecting.dmi',locate(start.x,start.y,start.z),"2")
		src.imageslist += new/image ('Selecting.dmi',end,"2")
		src.imageslist += new/image ('Selecting.dmi',locate(start.x,start.y+yd,start.z),"2")
		src.imageslist += new/image ('Selecting.dmi',locate(start.x+xd,start.y,start.z),"2")

		for(var/image/i in src.imageslist)

			src << i

	proc/endGrid()

		var

			// variables used to determine selection area
			max_X=max(start.x,end.x)
			min_X=min(start.x,end.x)
			max_Y=max(start.y,end.y)
			min_Y=min(start.y,end.y)

		// selection box is deleted
		for(var/image/i in src.imageslist)

			del(i)

		// turfs in box are checked for units
		for(var/turf/t in block(locate(min_X,min_Y,1),locate(max_X,max_Y,1)))

			// if you want to enable selecting more then one unit per turf, use a for() loop
			var/mob/unit/m = locate() in t

			if(m)

				// a unit was found at this location, select it
				var/image/i = new('Selecting.dmi',m,"1")
				select+=m
				src.imageslist+=i
				src << i

		start = null
		end = null

